#ifndef FREERDP_BUILD_CONFIG_H
#define FREERDP_BUILD_CONFIG_H

#define FREERDP_DATA_PATH "/usr/local/share/freerdp2"
#define FREERDP_KEYMAP_PATH ""
#define FREERDP_PLUGIN_PATH "lib/freerdp2"

#define FREERDP_INSTALL_PREFIX "/usr/local"

#define FREERDP_LIBRARY_PATH "lib"

#define FREERDP_ADDIN_PATH "lib/freerdp2"

#define FREERDP_SHARED_LIBRARY_SUFFIX ".so"
#define FREERDP_SHARED_LIBRARY_PREFIX  "lib"

#define FREERDP_VENDOR_STRING "FreeRDP"
#define FREERDP_PRODUCT_STRING "FreeRDP"

#define FREERDP_PROXY_PLUGINDIR "/home/pea/pobj/freerdp-2.4.0/build-mips64el/server/proxy/plugins"

#endif /* FREERDP_BUILD_CONFIG_H */
